/* Elvira Bluetooth FTP 
 * Copyright (C) 2007  Alberto Salas Bragado
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fs_local.h"

int local_list(char *directory, file_t **lista) {
	
	extern int alphasort();

	int selecc_arch(const struct dirent *entry) {
		if ((strcmp(entry->d_name, ".") == 0) || (strcmp(entry->d_name, "..") == 0))
			return (0);
		else
			return (1);
	}

	int nficheros, indice, res;
	struct dirent **ficheros;
	struct stat info;
	file_t *list = NULL;
	char temp_route[MAXPATHLEN];

	//Obtenemos la lista de subdirectorios y ficheros
	nficheros = scandir(directory, &ficheros, selecc_arch, alphasort);

	//Si ocurrio algun error
	if (nficheros < 0) {
		
		free(ficheros);
		*lista = NULL;
		return -1;
		
	// Si no se encontraron archivos
	} else if (nficheros == 0) {

		free(ficheros);
		*lista = NULL;
		return 0;
		
	}
	
	//Recorremos la lista
	for (indice=0; indice<nficheros; indice++) {

		//Reservamos memoria
		list = (file_t *) realloc(list, sizeof(file_t)*(indice+1));
		//Obtenemos el nombre
		strncpy(list[indice].name, ficheros[indice]->d_name, sizeof(list[indice].name)-1);
		//Tipo
		if (ficheros[indice]->d_type == DT_DIR)
			list[indice].type = FILET_FOLDER;
		else
			list[indice].type = FILET_FILE;
		
		//Obtenemos la ruta del fichero/directorio
		strcpy(temp_route, directory);
		if (strcmp(temp_route, "/") != 0)
			strcat(temp_route, "/");
		strcat(temp_route, list[indice].name);
		
		//Obtenemos el tamaño del fichero
		if (list[indice].type != FILET_FOLDER) {
			res = stat(temp_route, &info);
			if (res >= 0)
				sprintf(list[indice].size, "%u", (unsigned int) info.st_size);
			else
				strcpy(list[indice].size, "");
		} else {
			strcpy(list[indice].size, "");
		}
	}

	*lista = list;
	return nficheros;
	
}

int local_del_dir(char *name) {
	
	if (rmdir(name) < 0)
			
		return -1;
		
	return 0;
		
}

int local_del_file(char *name) {
	
	if (unlink(name) < 0)

		return -1;

	return 0;
	
}

int local_mkdir(char *name) {
	
	if (mkdir(name, 0755) < 0)

		return -1;

	return 0;
	
}

int local_file_exist(char *name) {
	
	if (access(name, F_OK) < 0)
		
		return -1;
	
	return 0;
	
}

int local_rperm(char *name) {
	
	if (access(name, R_OK) < 0)
		
		return -1;
	
	return 0;
	
}

int local_wperm(char *name) {
	
	if (access(name, W_OK) < 0)
		
		return -1;
	
	return 0;
	
}

int local_xperm(char *name) {
	
	if (access(name, X_OK) < 0)
		
		return -1;
	
	return 0;
	
}
